//***************************************************************************
//* Copyright (c) 2019 Saint Petersburg State University
//* All Rights Reserved
//* See file LICENSE for details.
//***************************************************************************

#pragma once

#include "pipeline/configs/aligner_config.hpp"
#include "modules/alignment/bwa_index.hpp"

namespace sensitive_aligner {

union DijkstraReturnCode {
    struct {
        bool queue_limit: 1;
        bool iter_limit: 1;
        bool not_connected: 1;
        bool no_path: 1;
        bool long_gap: 1;
        bool wide_gap: 1;
    };
    unsigned status = 0;
};

struct DijkstraParams {
    size_t queue_limit = 1000000;
    size_t iteration_limit = 1000000;
    size_t updates_limit = 1000000;
    bool find_shortest_path = false;
    bool restore_mapping = false;
    float penalty_ratio = 200;

    int max_ed_proportion = 3;
    int ed_lower_bound = 500;
    int ed_upper_bound = 2000;
};

struct GapClosingConfig: public DijkstraParams {
    bool run_dijkstra = true;
    size_t max_gs_states = 120000000;
};

struct EndsClosingConfig: public DijkstraParams {
    int max_restorable_length = 5000;
};

struct ProteinAlignmentConfig: public DijkstraParams {
    double min_alignment_len = 0.8;
    std::string penalty_matrix = "blosum90";
    bool stop_codon = true;
    int max_restorable_length = 5000;
    int indel_score = 5;
};

struct HitExtensionConfig {
    GapClosingConfig gap_cfg;
    EndsClosingConfig ends_cfg;
    ProteinAlignmentConfig protein_cfg;


    HitExtensionConfig(const GapClosingConfig &gap_cfg_ = GapClosingConfig(),
                   const EndsClosingConfig &ends_cfg_ = EndsClosingConfig(),
                   const ProteinAlignmentConfig &protein_cfg_ = ProteinAlignmentConfig())
    :gap_cfg(gap_cfg_),
     ends_cfg(ends_cfg_),
     protein_cfg(protein_cfg_)
    {}

};

} // namespace sensitive_aligner