# SPAligner

SPAligner performs fast and accurate alignment of molecular sequences to assembly graphs.

It can be used to align both nucleotide and amino acid sequences, e.g. long reads, coding sequences and proteins.

For more details on algorithm and tesing  please read our [pre-print on biorxiv](https://www.biorxiv.org/content/10.1101/744755v1).

## <a name="started"></a>TL;DR

### <a name="running"></a>Test run

Test graph and query sequences can be found in assembler/src/projects/spaligner/test_data/

For long-read alignment (`-d pacbio/nanopore`)

    cd assembler/src/projects/spaligner/test_data/
    [PATH_TO_BINARY]/spaligner -d pacbio -g graph.gfa -s pacbio_read.fastq.gz > aln.tsv

Alignments will be saved to tab-separated aln.tsv file. Each line represents an alignment information for a single query sequence (see [Results interpretation](#interpretation) for more details).

For amino acid sequences alignment (`-d protein`)
    
    cd assembler/src/projects/spaligner/test_data/
    [PATH_TO_BINARY]/spaligner -d protein -g graph.gfa -s protein.fasta -f aln.fasta -p aln_protein.fasta -o aln.tsv

This command will produce following output

    aln.tsv              tab-separated file with alignments information, each line represents an alignment of a single sequence (optonal)
    aln.fasta            each record represents alignment of a sequence onto assembly graph (optional)
    aln_protein.fasta    each record represents translated alignment of a sequence onto assembly graph (optional)


### <a name="input"></a>Input requirements

SPAligner accepts assembly graphs in [GFA1 format](https://github.com/GFA-spec/GFA-spec/blob/master/GFA1.md) from widely used short-read assemblers (e.g. SPAdes, Minia, Megahit) and tools for de Bruijn graph construction (e.g. BCALM2 or gbuilder from SPAdes distribution).


In general, SPALigner imposes the following major restriction on the graph structure: The overlap sizes between segments in GFA file (given by “L” records) must be explicitly provided and have the same value. Moreover, current implementation requires overlap sizes to have an odd value. Note that while providing an integer K as a parameter for Minia and BCALM2, the resulting size of the overlaps is K-1, whereas for SPAdes and Megahit it is K.


### <a name="output"></a>Output

By default, SPAligner outputs the resulting alignments in a tsv format to stdout. Alternatively, an output file can be provided with a `-o` option. If a FASTA file is provided with `-f` option, SPAligner will additionally save to the FASTA file sequences corresponding to the alignment paths. If for protein alignments a FASTA file is provided with `-p` option, SPAligner will save to the FASTA file amino acid sequences.

## Table of Contents
- [TL;DR](#started)
    - [Test run](#running)
    - [Input requirements](#input)
    - [Output](#output)
- [Compilation](#compilation)
- [Synopsis](#synopsis)
- [Results interpretation](#interpretation)
- [Algorithm overview](#algorithm)
    - [Long nucleotide sequence alignment pipeline](#longnuc)
    - [Amino-acid sequence alignment pipeline](#longaa)
- [Internal parameters](#parameters)
    - [Nucleotide sequence alignment](#nucparams)
    - [Amino acid sequence alignment](#aaparams)
    - [Internal run parameters](#dparams)
- [Contacts](#contacts)

## <a name="compilation"></a>Compilation

    git clone https://github.com/ablab/spades.git
    cd spades/assembler/
    mkdir build && cd build && cmake ../src
    make spaligner

Now SPAligner binary can be found in

    assembler/build/bin/spaligner

## <a name="synopsis"></a>Synopsis

Command line:

    spaligner -d <type> -s <fasta> -g <graph> [-t <value>] [-c <cfg>] [-f <filename>] [-p <filename>] [-o <filename>]

Parameters are:

`-d <type> `
    sequence type: nanopore, pacbio or protein

`-s <fasta> `
    file with query sequences in FASTA/FASTQ (gzip archives supported)

`-g <graph> `
    file with nucleotide sequence graph (in GFA1 format, see [Input requirements](#input))

`-t <value> `
    number of threads (optional, default: 8)

`-c <cfg> `
    config with [internal algorithm parameters](#parameters) (optional)

`-f <filename>`
    file to output nucleotide sequences corresponding to alignment paths in FASTA format (optional)

`-p <filename>`
    file to output protein sequences corresponding to alignment paths (optional). Only compatible with -d protein

`-o <filename> `
    TSV output file (optional, default: stdout)

If config `-c` with internal parameters is not set, SPAligner runs with default parameter values.


## <a name="interpretation"></a>Results interpretation

**TSV output format**

Each line in tsv-file represents alignments of a single sequence.


```
Read1    0      2491  536  1142  2491	44+,24+,22+,1+,38-	909,4,115,1,1142	AAACTTT...
```

|Col|Type  |Description                                                      |
|--:|:----:|:----------------------------------------------------------------|
|1  |string|Query sequence name                                              |
|2  |int   |Alignment path                                                   |
|3  |int   |Start position of alignment on query                             |
|4  |int   |End position of alignment on query                               |
|5  |int   |Start position of alignment on graph segment                     |
|6  |int   |End position of alignment on graph segment                       |
|7  |int   |Query sequence length                                            |
|8  |string|Alignment path                                                   |
|9  |string|Lengths of alignments on corresponding segments of alignment path|
|10 |string|Alignment sequence                                               |



**TSV output format (splitted alignments)**<br/>

SPAligner was primarily designed for finding semi-global alignments of query sequences. But if a suitable consecutive paths could not be found in the graph, SPAligner may produce split-read alignments, i.e. separate alignments covering non-overlapping segments of the query sequence (see Supplement Section ''Support for split-read alignments'' of the paper).

In this case information about the distinct alignments of all query regions is still summarized on a single line with the fields corresponding to individual alignments (start/end positions, paths, etc) joined via the `;` delimiter.


```
Read2     4;10  7;19       2;7   5;6       19    123+;288-,128+       3;3,6 GAT;TTATCCGGG
```

The line above encodes a split-read alignment of query `Read2`:

1. The first alignment starts on sequence on position 4 and ends on position 7.
Corresponding path consists of a single segment 123+ (i.e. 123+) with start on position 2 and end on position 5.
Path sequence: GAT.
2. While the second alignment covers the end of the sequence, starting on sequence on position 10 and ending on position 19.
Corresponding path consists of two segments 288- and 128+ (i.e. 288-,128+) with start on position 7 (on 288-) and end on position 6 (on 128+).
Path sequence: TTATCCGGG.


**Fasta output format**

The header of each record in fasta files shows information about alignment position on graph and on sequence.

```
>Read3|Segments=1-,2+,5+|start_g=3283|end_g=35|start_s=0|end_s=291|score=570|type=EXACT
ATGAAAATCACTCCTGAACAGGCTCGTGAGGCTCTGGATGCCTGGATATGTCGACCAGGAATGACACAGGAGCAGGCGACGATATTAATCACTGAAGCATTCTGGGCTTTGAAAGAGCGCCCGAACATCGATGTTCAGCGTGTCACATATGAAGGTGGCGCGATTGATCAGCGAGCGCTTGGCGTTAATCGAGTGAAGATATTTGAACGCTGGAAGGCTATCGACACCAGGGATAAGC
GTGAAAAGTTCACGGCGCTAGTGCCTGCAATTATGGAGGCTACCACTGGATGA
```

|Item    |Type  |Description                                                            |
|-------:|:----:|:----------------------------------------------------------------------|
|Read3   |string|Query sequence name                                                    |
|Segments|string|Alignment path                                                         |
|start_g |int   |Start position of alignment on graph segment                           |
|end_g   |int   |End position of alignment on graph segment                             |
|start_s |int   |Start position of alignment on query                                   |
|end_s   |int   |End position of alignment on query                                     |
|score   |int   |Alignment score (only for `-d protein`)                                |
|type    |string|Type of resulting alignment: EXACT/PARTIAL/FULL (only for `-d protein`)|

EXACT if amino acid sequence is identical to amino acid alignment sequence, FULL if sequence aligned fully but not indetically and PARTIAL if only a part of sequence aligned (only for amino acid alignment).


## <a name="algorithm"></a>Algorithm overview

We decided to save GFA1-related notation and use segments ("S") in algorithm description below.
Despite the fact that in our pre-print text the notation is sligthly different as it uses graph theory terms (such as edges and vertices),
one can easly match these two notations.

### <a name="longnuc"></a>Long nucleotide sequence alignment pipeline

Overview of the alignment of the nucleotide query sequence *S* (orange bar) to assembly graph *G*. Assembly graph segments are considered directed left-to-right (explicit segment orientation was omitted to improve the clarity).

![pipeline](pipeline.jpg)

1. **Anchor search.** Anchors (regions of high similarity) between the query and the segment labels are identified with [BWA-MEM](http://bio-bwa.sourceforge.net/). 
2. **Anchor filtering.** Anchors shorter than *K*, assembly graph *K*-mer size,(anchors 2, 6, 11), anchors “in the middle” of long segment (anchor 7) or ambiguous anchors (anchor 10 mostly covered by anchor 9, both anchors 4 and 5) are discarded.
3. **Anchor chaining.** Heaviest chain of compatible anchors (chain 3->8->9) is determined.
4. **Reconstruction of filling paths.** Paths for fragments of the query between the consecutive chain anchors (as well as left- and right-most fragments) are reconstructed. The procedure is performed using fast library for sequence alignment [Edlib](https://github.com/Martinsos/edlib).

### <a name="longaa"></a>Amino-acid sequence alignment pipeline

Overview of the alignment of the amino-acid query sequence *S* (orange bar) to assembly graph *G*. Assembly graph segments are considered directed left-to-right (explicit segment orientation was omitted to improve the clarity).

![pipeline_protein](pipeline_protein.jpg)

1. **Anchor search.** Anchors (regions of high similarity) between the query and the segment labels are identified with [BWA-MEM](http://bio-bwa.sourceforge.net/). 
2. **Anchor filtering.** Anchors shorter than *K*, assembly graph *K*-mer size,(anchors 1, 4, 8), anchors with a wrong frame shift (anchors 2, 7, 9) are discarded.
3. **Alignment extension.** Search of an optimal alignments extending each of the remaining anchors.
4. **Alignment scoring.** Obtained alignment paths are re-scored via [Parasail library](https://github.com/jeffdaily/parasail) for fast amino acid sequence alignment and are stored to resulting fasta-file.

## <a name="parameters"></a>Internal parameters

Full list of parameters can be found in spaligner_config.yaml. 

### <a name="nucparams"></a> Nucleotide sequence alignment

* `run_dijkstra: true` Run Dijkstra algorithm to find alignment between anchors, if `run_dijkstra: false`, SPAligner will check limited number of paths and return the best one.
* `restore_ends: true` Restore alignment path before leftmost anchor and after rightmost anchor.
* `internal_length_cutoff: 200` Anchors with length < `internal_length_cutoff` will be filtered out.
* `path_limit_stretching: 1.3` Pair of anchors is considered to be compatible if (the minimal distance between them in graph) 
                                < `path_limit_stretching` * (the distance between their positions on sequence).
* `path_limit_pressing: 0.7` Pair of anchors is considered to be compatible if (the minimal distance between them in graph) 
                                > `path_limit_pressing` * (the distance between their positions on sequence).
* `max_path_in_chaining: 15000` Limit on number of paths to consider between two anchors on anchors chaining step.
* `max_vertex_in_chaining: 5000` Limit on number of vertices to consider between two anchors on anchors chaining step.

### <a name="aaparams"></a>Amino acid sequence alignment

* `stop_codon: true` Algorithm doesn't extend alignment from states with stop codon triplets.
* `min_alignment_len: 0.8` Return only alignment with `length > min_alignment_len * (query sequence length)`.
* `penalty_matrix: blosum62` Substitution matrix name (supports a number of [BLOSUM and PAM matrices](https://github.com/jeffdaily/parasail/tree/master/parasail/matrices)).
* `indel_score: 5` Penalty for insertions and deletions equals to `(max value in penalty_matrix) + indel_score`.


### <a name="dparams"></a>Internal run parameters

* `queue_limit: 1000000` Limit on queue length. 
* `iteration_limit: 1000000` Limit on total number of queue extraction. 
* `updates_limit: 1000000` Limit on number of updates of shortest distance for all states.
* `find_shortest_path: true` If `find_shortest_path: false` Dijkstra algorithm will stop when it reaches finish state without searching for shortest path.
* `restore_mapping: false` If `restore_mapping: true` Dijkstra algorithm will return full alignment information (used by for developers).
* `penalty_ratio: 0.1` Algorithm never considers states representing an alignment of a prefix S[0:i] with score more than min_score(i) + i*penalty_ratio (for nucleotide sequence alignment only).
* `max_ed_proportion: 3` Maximal edit distance is bounded by a fraction of the query sequence length |S|/max_ed_proportion. Increase of max_ed_proportion leads to shorter alignments but with higer identity.
* `ed_lower_bound: 500` Minimal penalty score of alignment.
* `ed_upper_bound: 2000` Maximal penalty score of alignment.
* `max_gs_states: 120000000` If number of queue states exceeds `max_gs_limit` then shortest path search is not performed (for nucleotide sequence alignment only).
* `max_restorable_length: 5000` If distance between two anchors or between leftmost/rightmost anchor and start/end exceeds `max_restorable_length` then shortest path search is not performed.

Increase of `max_gs_states`, `max_restorable_length`, `queue_limit`, `iteration_limit` or `updates_limit` may lead to longer alignments with the same identity level, but slows down the process and can use much more memory. Please change them if you 100% confident in what you are doing.

Turning off `restore_ends` or `run_dijkstra` in nucleotide sequence alignment mode leads to shorter alignments, but considerable speed-up.

## <a name="contacts"></a> Contacts

For any questions or suggestions please do not hesitate to contact Tatiana Dvorkina <tedvorkina@gmail.com>.